import {attribute, DataStore, model, Model, Transport, Types} from '@rx-suite/data';
import {i18n} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {action, computed, observable, runInAction, when} from "mobx";
import {DashboardTargetModel, DashboardTargetStore} from "./DashboardTargetStore";
import {computedFn} from "mobx-utils";
import {CustomTarget} from "./Types";
import {v4 as uuidv4} from 'uuid';

class DashboardComponentResultTransport<T extends Model> extends Transport<T> {
    async run(id: string) {
        const resp = await this.fetch(`/${id}/run`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            }
        });
        return resp.json();
    }
}

@model({
    transportLayer: new DashboardComponentResultTransport("/api/dashboard_components", "dashboard_component"),
    permissionTarget: "DashboardComponent",
    routingInfo: {
        create: "/dashboard_components/create",
        update: "/dashboard_components/:id/edit",
        read: "/dashboard_components/:id/read"
    }
})
export class DashboardComponentModel extends Model {
    types: Types = {
        parameters: 'json',
        display_options: 'json',
        data: 'json',
    };

    @attribute({
        label: i18n._(t`Name`),
        description: i18n._(t`This is how you will identify the component in lists (for example, you can search for this name).  It will also be shown as a title on some component types.`),
        validation: yup.string().required(i18n._(t`You must give your component a name`))
    })
    name: string = '';

    @attribute({
        label: i18n._(t`Description`),
        description: i18n._(t`Add any notes here.  The information will not be included in the dashboard display - it is for reference purposes only.`),
        validation: yup.string()
    })
    desc: string = '';

    @attribute({
        label: i18n._(t`Run As`),
        description: i18n._(t`Run the dashboard using this user's credentials`),
        validation: yup.string().nullable()
    })
    run_user_id: string = '';

    @attribute({
        label: i18n._(t`Refresh Interval (Seconds)`),
        description: i18n._(t`Control how often the component fetches fresh data.`),
        validation: yup.number().min(1, i18n._(t`Choose a larger refresh interval`)).required(i18n._(t`You must give your component a refresh interval`))
    })
    refresh_interval: number = 60;

    @attribute({
        label: i18n._(t`Parameters`),
        validation: yup.string(),
        autoTable: false
    })
    parameters: any = observable({});

    @attribute({
        label: i18n._(t`Display Options`),
        validation: yup.string().nullable(),
        autoTable: false
    })
    display_options: any = observable({});

    @attribute({
        label: i18n._(t`Display Type`),
        validation: yup.string().when('dashboard_target_id', (dashboard_target_id: string | undefined, schema: any) => {
            if (dashboard_target_id) {
                const target = DashboardTargetStore.get(dashboard_target_id.toString()) as DashboardTargetModel;
                if (target && target.displayTypes.length) {
                    return schema.required(`You must choose a display type`);
                }
            }
            return schema;
        }),
        autoTable: false
    })
    display_type: string = '';

    @attribute({
        label: i18n._(t`Data`),
        autoTable: false, autoForm: false
    })
    data: any = observable({});

    @attribute({
        label: i18n._(t`Last Run Error`),
        validation: yup.string().nullable(),
        autoForm: false
    })
    last_run_error: string | null = null;

    @attribute({
        label: i18n._(t`Target`),
        connects: 'DashboardTargetModel',
        description: i18n._(t`The selected target determines what kinds of data can be fetched and what options you have to display it.  Available targets depend on your installed Rx-Suite modules.`),
        validation: yup.number().required(i18n._(t`You must set a dashboard target`)),
        autoTable: false
    })
    dashboard_target_id: number | undefined = undefined;

    @attribute({label: i18n._(t`Created At`), autoForm: false})
    created_at: Date = new Date();

    @attribute({label: i18n._(t`Updated At`), autoForm: false})
    updated_at: Date = new Date();

    @computed
    get displayTypes() {
        if (this.viewModel.dashboard_target) {
            return (this.viewModel.dashboard_target as DashboardTargetModel).displayTypes;
        }
        return [];
    }

    @computed
    get displayTypeName() {
        const typeDef = this.displayTypes.find(t => t.value === this.display_type);
        if (typeDef) {
            return typeDef.label;
        } else {
            return this.display_type || ''
        }
    }

    isValidDisplayType = computedFn(function getDef(this: DashboardComponentModel, type: string): any {
        return this.displayTypes && (this.displayTypes.length > 0) && this.displayTypes.find(t => t.value === type);
    });

    targetParameterName = computedFn(function getDef(this: DashboardComponentModel, value: string): any {
        if (this.viewModel.parameters && this.viewModel.parameters.custom_targets) {
            const custom = this.viewModel.parameters.custom_targets.find((c: CustomTarget) => c.id === value);
            if (custom) {
                return custom.label;
            }
        }

        const dashboardTarget = this.viewModel.dashboard_target as DashboardTargetModel;
        if (dashboardTarget && dashboardTarget.parameters && dashboardTarget.parameters.targets) {
            const returnDef = (dashboardTarget.parameters.targets.value_list || []).find(t => t.value === value);
            return (returnDef && returnDef.label) ? returnDef.label : value;
        } else {
            return value;
        }
    });

    targetParameterProp = computedFn(function getDef(this: DashboardComponentModel, value: string, prop: string): any {
        const dashboardTarget = this.viewModel.dashboard_target as DashboardTargetModel;
        if (dashboardTarget && dashboardTarget.parameters && dashboardTarget.parameters.targets) {
            const returnDef = (dashboardTarget.parameters.targets.value_list || []).find(t => t.value === value) as any;
            if (returnDef) {
                return (returnDef[prop] === undefined) ? null : returnDef[prop];
            } else {
                return null;
            }
        } else {
            return null;
        }
    });

    @action.bound
    resetParameters() {
        this.nestedAssignment('parameters', {});
        if (!this.isValidDisplayType(this.viewModel.display_type)) {
            if (this.displayTypes.length === 1) {
                this.nestedAssignment('display_type', this.displayTypes[0].value);
            } else {
                this.nestedAssignment('display_type', '');
            }
        }
        if (this.viewModel.dashboard_target) {
            Object.values((this.viewModel.dashboard_target as DashboardTargetModel).parameters).forEach(parameter => {
                if (parameter.default) {
                    runInAction(() => {
                        this.nestedAssignment(`parameters.${parameter.id}`, parameter.default)
                    });
                }
            });
        }
        this.nestedAssignment('data', {})
    }

    @action.bound
    pushCustomTarget() {
        this.nestedAssignment('parameters.custom_targets', [...((this.viewModel.parameters || {}).custom_targets || []), {
            id: uuidv4(),
            label: "",
            combines: [],
        } as CustomTarget])
    }

    @action.bound
    removeCustomTarget(target: CustomTarget) {
        this.nestedAssignment('parameters.custom_targets', ((this.viewModel.parameters || {}).custom_targets || []).filter((t: CustomTarget) => t !== target));
    }


    @computed get targetIds() {
        const params = this.viewModel.parameters || {};
        return [...(params.targets || []),
            ...(params.custom_targets || []).map((c: CustomTarget) => c.id)
        ]
    }

    @computed
    get exportName() {
        return this.name || this.getId() as string;
    }

    updateData = async () => {
        const transport = this.store.transportLayer as DashboardComponentResultTransport<DashboardComponentModel>;
        return transport.run(this.getId());
    }

    asExport(): object {
        const standardDump = super.asExport() as any;
        const target = this.dashboard_target as DashboardTargetModel;
        const result = {
            dashboard_target: {
                module_id: target.module_id,
                dashboard_id: target.dashboard_id,
            }
        } as any;
        Object.keys(standardDump).forEach(k => {
            if (k !== 'dashboard_target_id') {
                result[k] = standardDump[k]
            }
        });
        return result;
    }

    /**
     * Inverse of asExport
     * @returns {object} An export-friendly object
     */
    @action
    fromExport(json: Model) {
        this.updateFromJson(json, true);
        if (json.dashboard_target) {
            when(() => DashboardTargetStore.models.length > 0, () => {
                runInAction(() => {
                    const target_data = json.dashboard_target as { module_id?: string, dashboard_id?: string };
                    const target = DashboardTargetStore.models.find(t => t.module_id === target_data.module_id && t.dashboard_id === target_data.dashboard_id);
                    if (target) {
                        this.dashboard_target_id = target.id as number;
                    }
                })
            });
        }
    }

    @computed
    get typographyTitleStyle() {
        const style = {
            marginLeft: '16px', marginRight: '16px'
        } as any;
        const position = ((this.viewModel.display_options || {}).title || {}).position;
        if (!position || (position === 'top-left')) {
            style.textAlign = 'left';
        } else if (position === 'top-middle') {
            style.textAlign = 'center';
        } else if (position === 'top-right') {
            style.textAlign = 'right';
        } else if (position === 'omit') {
            style.display = 'none';
        }
        return style;
    }

    @computed
    get graphOptions() {
        const options = {} as any;

        if ((((this.viewModel.display_options || {}).title || {}).position === 'top-left') || (((this.viewModel.display_options || {}).title || {}).position === undefined)) {
            options.title = {};
            options.title.align = 'left';
            options.title.text = this.viewModel.name;
        } else if (((this.viewModel.display_options || {}).title || {}).position === 'top-middle') {
            options.title = {};
            options.title.align = 'center';
            options.title.text = this.viewModel.name;
        } else if (((this.viewModel.display_options || {}).title || {}).position === 'top-right') {
            options.title = {};
            options.title.align = 'right';
            options.title.text = this.viewModel.name;
        }

        return options;
    }
}

export const DashboardComponentStore = DashboardComponentModel.store as unknown as DataStore<DashboardComponentModel>;
DashboardComponentStore.setLimit(10000);