import {attribute, DataStore, model, Model, Transport} from '@rx-suite/data';
import {i18n} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {action, computed, observable} from "mobx";
import {DashboardGroupDashboardModel, DashboardGroupDashboardStore} from "./DashboardGroupDashboardStore";
import {DashboardModel} from "./DashboardStore";
import {computedFn} from "mobx-utils";

@model({
    transportLayer: new Transport("/api/dashboard_groups", "dashboard_group"),
    permissionTarget: "DashboardGroup",
    routingInfo: {
        create: "/dashboard_groups/create",
        update: "/dashboard_groups/:id/edit",
        read: "/dashboard_groups/:id/read"
    }
})
export class DashboardGroupModel extends Model {
    @attribute({
        label: i18n._(t`Name`),
        description: i18n._(t`This is how you will identify the dashboard in lists (for example, you can search for this name).`),
        validation: yup.string().required(i18n._(t`You must give your dashboard a name`))
    })
    name: string = '';

    @attribute({
        label: i18n._(t`Description`),
        description: i18n._(t`Add any notes here.  The information will not be included in the dashboard display - it is for reference purposes only.`),
        validation: yup.string()
    })
    desc: string = '';

    @observable
    tmpMembers: DashboardGroupDashboardModel[] = [];

    @observable
    memberDeletionQueue: DashboardGroupDashboardModel[] = [];

    @computed
    get members() {
        const typedID = parseInt(this.getId(), 10);
        return (DashboardGroupDashboardStore.models.filter(member => {
            return (member.viewModel.dashboard_group_id === typedID) && !(this.memberDeletionQueue.find(p => p.getId() === member.getId()))
        }) as DashboardGroupDashboardModel[]).concat(this.tmpMembers).sort((a, b) => {
            return (a.viewModel.rank || 0) - (b.viewModel.rank || 0)
        });
    }

    @computed
    get dashboards() {
        return this.members.map(m => m.viewModel.dashboard) as DashboardModel[];
    }

    hasDashboard = computedFn(function check(this: DashboardGroupModel, dashboard: DashboardModel): any {
        return !!this.dashboards.find(d => d && (d.getId() === dashboard.getId()))
    });

    nextMember = computedFn(function check(this: DashboardGroupModel, currentMember: DashboardGroupDashboardModel): any {
        let currentPosition = this.members.findIndex(m => m.getId() === currentMember.getId());
        currentPosition++;
        if (currentPosition >= this.members.length) {
            currentPosition = 0;
        }
        return this.members[currentPosition] as DashboardGroupDashboardModel;
    });

    previousMember = computedFn(function check(this: DashboardGroupModel, currentMember: DashboardGroupDashboardModel): any {
        let currentPosition = this.members.findIndex(m => m.getId() === currentMember.getId());
        currentPosition--;
        if (currentPosition < 0) {
            currentPosition = this.members.length - 1;
        }
        return this.members[currentPosition] as DashboardGroupDashboardModel;
    });

    @action.bound
    addDashboard(dashboard: DashboardModel) {
        const member = DashboardGroupDashboardStore.createModel();
        member.nestedAssignment('dashboard_group_id', parseInt(this.getId(), 10));
        member.nestedAssignment('dashboard_id', parseInt(dashboard.getId(), 10));
        member.nestedAssignment('rank', this.members.length);
        member.nestedAssignment('interval', 15);
        this.tmpMembers.push(member);
        this.memberDeletionQueue = this.memberDeletionQueue.filter(m => m.getId() !== member.getId());
        this.normalizeRanks();
    }

    @action.bound
    normalizeRanks() {
        this.members.forEach((m, i) => {
            m.nestedAssignment('rank', i);
        })
    }

    @action.bound
    removeDashboard(dashboard: DashboardModel) {
        this.members.forEach(m => {
            if (m.viewModel.dashboard_id && (m.viewModel.dashboard_id.toString() === dashboard.getId().toString())) {
                this.removeMember(m);
            }
        });
        this.normalizeRanks();
    }

    @action.bound
    removeMember(member: DashboardGroupDashboardModel) {
        this.members.forEach(m => {
            if (m.getId() === member.getId()) {
                this.tmpMembers = this.tmpMembers.filter(mTmp => mTmp.getId() !== m.getId());
                if (!m.isNew) {
                    this.memberDeletionQueue.push(m);
                }
            }
        });
    }

    @action.bound
    async commit() {
        const resp = await super.commit();
        this.members.forEach(m => {
            m.nestedAssignment('dashboard_group_id', resp.id);
            m.commit();
        });
        this.memberDeletionQueue.forEach(m => {
            m.delete();
        });
        this.tmpMembers = [];
        return resp;
    }
}

export const DashboardGroupStore = DashboardGroupModel.store as DataStore<DashboardGroupModel>;
DashboardGroupStore.setLimit(10000);

class SlideshowState {
    @observable
    timerId: NodeJS.Timeout | null = null;

    @observable
    memberId: string | null = null;

    @action.bound play(currentMember: DashboardGroupDashboardModel) {
        this.pause();
        this.memberId = currentMember.getId();
        this.timerId = setTimeout(() => {
            this.goNext();
        }, currentMember.interval * 1000);
    }

    @action.bound goNext() {
        if (this.memberId) {
            const currentMember = DashboardGroupDashboardStore.get(this.memberId);
            if (currentMember) {
                const group = currentMember.dashboard_group as DashboardGroupModel;
                if (group) {
                    this.memberId = group.nextMember(currentMember).getId();
                    if (this.playing) {
                        this.play(group.nextMember(currentMember));
                    }
                }
            }
        }
    }

    @action.bound goBack() {
        if (this.memberId) {
            const currentMember = DashboardGroupDashboardStore.get(this.memberId);
            if (currentMember) {
                const group = currentMember.dashboard_group as DashboardGroupModel;
                if (group) {
                    this.memberId = group.previousMember(currentMember).getId();
                    if (this.playing) {
                        this.play(group.previousMember(currentMember));
                    }
                }
            }
        }
    }

    @action.bound pause() {
        if (this.timerId) {
            clearTimeout(this.timerId);
        }
        this.timerId = null;
    }

    @computed
    get playing() {
        return !!this.timerId;
    }
}

export const slideshowState = new SlideshowState();